/***************************************************************
 * Name:      fretlessMain.cpp
 * Purpose:   Code for Application Frame
 * Author:    John Conner Atkeson ()
 * Created:   2007-12-02
 * Copyright: John Conner Atkeson (http://jca3.freeshell.org)
 * License:
 **************************************************************/

#include "wx_pch.h"
#include "fretlessMain.h"
#include <wx/msgdlg.h>

//(*InternalHeaders(fretlessFrame)
#include <wx/bitmap.h>
#include <wx/font.h>
#include <wx/fontenum.h>
#include <wx/fontmap.h>
#include <wx/image.h>
#include <wx/intl.h>
#include <wx/settings.h>
#include <wx/string.h>
//*)


#include "audio.h"
#include "drawing_canvas.h"
#include "waveform_canvas.h"

#define have_player_main 0


//helper functions
enum wxbuildinfoformat {
  short_f, long_f
};

wxString wxbuildinfo(wxbuildinfoformat format)
{
  wxString wxbuild(wxVERSION_STRING);

  if (format == long_f )
  {
#if defined(__WXMSW__)
    wxbuild << _T("-Windows");
#elif defined(__UNIX__)
    wxbuild << _T("-Linux");
#endif

#if wxUSE_UNICODE
    wxbuild << _T("-Unicode build");
#else
    wxbuild << _T("-ANSI build");
#endif // wxUSE_UNICODE
  }

  return wxbuild;
}

//(*IdInit(fretlessFrame)
const long fretlessFrame::ID_SLIDER1 = wxNewId();
const long fretlessFrame::ID_PANEL1 = wxNewId();
const long fretlessFrame::ID_BUTTON1 = wxNewId();
const long fretlessFrame::idMenuQuit = wxNewId();
const long fretlessFrame::idMenuAbout = wxNewId();
const long fretlessFrame::ID_STATUSBAR1 = wxNewId();
//*)

BEGIN_EVENT_TABLE(fretlessFrame,wxFrame)
  //(*EventTable(fretlessFrame)
  //*)
END_EVENT_TABLE()


#if have_player_main
Player_Class *player;
#endif
bool left_mouse_down;

fretlessFrame::fretlessFrame(wxWindow* parent,wxWindowID id)
{
  //(*Initialize(fretlessFrame)
  wxMenuBar* MenuBar1;
  wxMenu* Menu1;
  wxMenuItem* MenuItem1;
  wxMenu* Menu2;
  wxMenuItem* MenuItem2;

  Create(parent, id, wxEmptyString, wxDefaultPosition, wxDefaultSize, wxDEFAULT_FRAME_STYLE, _T("wxFrame"));
  SetClientSize(wxSize(509,308));
  BoxSizer1 = new wxBoxSizer(wxHORIZONTAL);
  Octave_Slider = new wxSlider(this, ID_SLIDER1, 0, 0, 100, wxDefaultPosition, wxDefaultSize, wxSL_VERTICAL|wxSL_INVERSE, wxDefaultValidator, _T("ID_SLIDER1"));

  BoxSizer1->Add(Octave_Slider, 0, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 0);
  Panel1 = new wxPanel(this, ID_PANEL1, wxDefaultPosition, wxSize(43,119), wxTAB_TRAVERSAL, _T("ID_PANEL1"));
  Button1 = new wxButton(Panel1, ID_BUTTON1, _("Label"), wxPoint(17,70), wxDefaultSize, 0, wxDefaultValidator, _T("ID_BUTTON1"));
  BoxSizer1->Add(Panel1, 0, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 0);
  SetSizer(BoxSizer1);
  MenuBar1 = new wxMenuBar();
  Menu1 = new wxMenu();
  MenuItem1 = new wxMenuItem(Menu1, idMenuQuit, _("Quit\tAlt-F4"), _("Quit the application"), wxITEM_NORMAL);
  Menu1->Append(MenuItem1);
  MenuBar1->Append(Menu1, _("&File"));
  Menu2 = new wxMenu();
  MenuItem2 = new wxMenuItem(Menu2, idMenuAbout, _("About\tF1"), _("Show info about this application"), wxITEM_NORMAL);
  Menu2->Append(MenuItem2);
  MenuBar1->Append(Menu2, _("Help"));
  SetMenuBar(MenuBar1);
  StatusBar1 = new wxStatusBar(this, ID_STATUSBAR1, 0, _T("ID_STATUSBAR1"));
  int StatusBar1__widths[1] = { -1 };
  int StatusBar1__styles[1] = { wxSB_NORMAL };
  StatusBar1->SetFieldsCount(1,StatusBar1__widths);
  StatusBar1->SetStatusStyles(1,StatusBar1__styles);
  SetStatusBar(StatusBar1);
  BoxSizer1->SetSizeHints(this);
  Connect(ID_SLIDER1,wxEVT_SCROLL_TOP|wxEVT_SCROLL_BOTTOM|wxEVT_SCROLL_LINEUP|wxEVT_SCROLL_LINEDOWN|wxEVT_SCROLL_PAGEUP|wxEVT_SCROLL_PAGEDOWN|wxEVT_SCROLL_THUMBTRACK|wxEVT_SCROLL_THUMBRELEASE|wxEVT_SCROLL_CHANGED,(wxObjectEventFunction)&fretlessFrame::OnOctave_SliderCmdScroll);
  Connect(ID_SLIDER1,wxEVT_SCROLL_CHANGED,(wxObjectEventFunction)&fretlessFrame::OnOctave_SliderCmdScrollChanged);
  Connect(ID_BUTTON1,wxEVT_COMMAND_BUTTON_CLICKED,(wxObjectEventFunction)&fretlessFrame::OnButton1Click);
  Connect(idMenuQuit,wxEVT_COMMAND_MENU_SELECTED,(wxObjectEventFunction)&fretlessFrame::OnQuit);
  Connect(idMenuAbout,wxEVT_COMMAND_MENU_SELECTED,(wxObjectEventFunction)&fretlessFrame::OnAbout);
  //*)
#if 1
  // wxMessageBox(wxT("This is the message."), wxT("This is the title"), wxICON_INFORMATION);
  // wxMessageBox(_T("Hello World!"));
  // wxMessageBox(wxT("Hello World!"));


  const long ID_PANEL_WE = wxNewId();
  Waveform_Editor_Class *WaveformEditor = new Waveform_Editor_Class(this, ID_PANEL_WE);
  BoxSizer1->Add(WaveformEditor, 0, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 0);

  const long ID_PANEL_DC = wxNewId();
  DrawingCanvasClass *DrawingCanvas = new DrawingCanvasClass(this, ID_PANEL_DC);
  BoxSizer1->Add(DrawingCanvas, 0, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 0);

  //SetSizer(BoxSizer1);
  BoxSizer1->Fit(this);
#endif
  left_mouse_down = true;
#if have_player_main
  player = new player_class();
#endif
  // DrawingCanvas->Painter();

  Octave_Slider->SetMin(700);
  Octave_Slider->SetMax(1300);
  DrawingCanvas->Voice_Track->Set_Signal_Generator(WaveformEditor->Waveform_Canvas->Voice_Track->sgen);
}

fretlessFrame::~fretlessFrame()
{
  //(*Destroy(fretlessFrame)
  //*)
}

void fretlessFrame::OnQuit(wxCommandEvent& event)
{
  Close();
}

void fretlessFrame::OnAbout(wxCommandEvent& event)
{
  wxString msg = wxbuildinfo(long_f);
  wxMessageBox(msg, _("Welcome to..."));
}

void fretlessFrame::OnOctave_SliderCmdScrollChanged(wxScrollEvent& event)
{
}

void fretlessFrame::OnOctave_SliderCmdScroll(wxScrollEvent& event)
{
}

void fretlessFrame::OnButton1Click(wxCommandEvent& event)
{
}
